/* "Image Tool"
 * Copyright (C) 2000-01 by Next Generation
 * Written by Chris Hansen.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * NEXT GENERATION on the Web: www.nextgeneration.dk
 * CONTACT: info@nextgeneration.dk
 */

#define WIN32_LEAN_AND_MEAN 1
#define _WIN32_IE 0x0501
#include <windows.h>
#include <commctrl.h>
#include "resource.h"
#include "verinfo.h"

#define IDM_SYS_ABOUT 10001

#define ID_EDITCHILD 50001
#define ID_CLOSE_BUTTON 50002
#define ID_INFO_BUTTON 50003
#define ID_RADIO_BUTTON1 50004
#define ID_RADIO_BUTTON2 50005
#define ID_RUN_BUTTON 50006
#define ID_SELECT_BUTTON 50007
#define ID_TIMER 50008

static int iTimer;
static BOOL bActive, bInfoWindow;
static HWND hwndButtonClose, hwndButtonInfo, hwndButtonRun, hwndButtonSelect, hwndEdit, hwndEditInfo, hwndRadio1, hwndRadio2, hwndStatus;
static PROCESS_INFORMATION pi;
static TCHAR sCurrentDir[MAX_PATH], sDestDrive[MAX_PATH], sExeDir[MAX_PATH], sFileName[MAX_PATH], sFullFileName[MAX_PATH], sHelpFile[MAX_PATH];

HINSTANCE hInst;

extern BOOL DoesImageExist (char *psFileName);
extern HFONT GetFont (const char *lpFont, long fSize, long fWeight);
extern void RunRawrite (LPPROCESS_INFORMATION lppi, HWND hEditInfo, char *psExeDir, char *psFullFileName, char *psDestDrive);
extern BOOL SelectFile (HWND hwnd, char *psTitleName, char *psCurrentDir);

BOOL CALLBACK DlgProcAbout (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK WndProcMain (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

int WINAPI
WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  HACCEL hAccel;
  HANDLE hMutex;
  HMENU hMenu;
  HWND hwnd;
  INITCOMMONCONTROLSEX icc;
  MSG msg;
  TCHAR sBuffer[100];
  WNDCLASS wndclass;
  
  /* Check whether this applications is running already */
  hMutex = CreateMutex (NULL, FALSE, RES_PROGRAM_INTERNAL);
  if (GetLastError () == ERROR_ALREADY_EXISTS)
    {
      CloseHandle (hMutex);
      wsprintf (sBuffer, "%s is already running.", RES_PROGRAM_NAME);
      MessageBox (NULL, sBuffer, RES_PROGRAM_NAME, MB_OK | MB_ICONINFORMATION);
      return 0;
    }
  
  /* Initialize some variables */
  bActive = FALSE;
  bInfoWindow = FALSE;

  /* Prepare Microsoft Common Controls */
  icc.dwSize = sizeof (INITCOMMONCONTROLSEX);
  icc.dwICC = ICC_BAR_CLASSES;
  InitCommonControlsEx (&icc);
  
  GetCurrentDirectory (MAX_PATH, sCurrentDir);
  lstrcpy (sExeDir, sCurrentDir);
  wsprintf (sHelpFile, "%s\\Imgtool.hlp", sCurrentDir);

  /* Prepare the main window class */
  wndclass.style = CS_HREDRAW | CS_VREDRAW;
  wndclass.lpfnWndProc = WndProcMain;
  wndclass.cbClsExtra = 0;
  wndclass.cbWndExtra = 0;
  wndclass.hInstance = hInstance;
  wndclass.hIcon = LoadIcon (hInstance, MAKEINTRESOURCE (IDI_ICON1));
  wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
  wndclass.hbrBackground = (HBRUSH) (COLOR_APPWORKSPACE);
  wndclass.lpszMenuName = NULL;
  wndclass.lpszClassName = RES_PROGRAM_INTERNAL;
  RegisterClass (&wndclass);
  
  /* Create the main window */
  wsprintf (sBuffer, "%s v%d.%d.%d", RES_PROGRAM_NAME, RES_PROGRAM_VERSION, RES_PROGRAM_MAJOR, RES_PROGRAM_MINOR);
  hwnd = CreateWindowEx (0, RES_PROGRAM_INTERNAL, sBuffer, WS_CAPTION | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_MINIMIZEBOX | WS_SYSMENU, (GetSystemMetrics (SM_CXSCREEN) - 250) / 2, (GetSystemMetrics (SM_CYSCREEN) - 182) / 2, 250, 182, NULL, /*hMenu*/NULL, hInstance, NULL);

  /* Insert 'about' item in the system menu */
  hMenu = GetSystemMenu (hwnd, FALSE);
  AppendMenu (hMenu, MF_SEPARATOR, 0, NULL);
  AppendMenu (hMenu, MF_STRING, IDM_SYS_ABOUT, "About...");
  SetMenuItemBitmaps (hMenu, GetMenuItemID (hMenu, 8), MF_BYCOMMAND, LoadBitmap (hInst, MAKEINTRESOURCE (IDB_ABOUT13x13)), NULL);

  ShowWindow (hwnd, nCmdShow);
  UpdateWindow (hwnd);

  /* Load accelerators */
  hAccel = LoadAccelerators (hInstance, MAKEINTRESOURCE (IDR_ACCELERATOR1));

  /* Start message loop for window procedure */
  while (GetMessage (&msg, NULL, 0, 0))
    {
      if (!TranslateAccelerator (hwnd, hAccel, &msg))
        {
          TranslateMessage (&msg);
          DispatchMessage (&msg);
        }
    }

  /* Remember to remove the mutex when the application returns */
  CloseHandle (hMutex);

  return msg.wParam;
}

LRESULT CALLBACK
WndProcMain (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  int iAnswer, iCount;
  DWORD dwExitCode;
  HDC hdc;
  LPCTSTR sWndText1 = "Image file:", sWndText2 = "Destination drive:";
  PAINTSTRUCT ps;
  RECT rc;
  TCHAR sStatusText[MAX_PATH];
  
  GetExitCodeProcess (pi.hProcess, &dwExitCode);
  if (bActive && dwExitCode != STILL_ACTIVE)
    {
      bActive = FALSE;
      EnableWindow (hwndEdit, TRUE);
      EnableWindow (hwndButtonSelect, TRUE);
      EnableWindow (hwndRadio1, TRUE);
      EnableWindow (hwndRadio2, TRUE);
      EnableWindow (hwndButtonClose, TRUE);
      EnableWindow (hwndButtonRun, TRUE);
      KillTimer (hwnd, ID_TIMER);
      SetWindowText (hwndEdit, NULL);
      SetWindowText (hwndEditInfo, "Done!");
      wsprintf (sStatusText, "%s (Press F1 for help)", RES_PROGRAM_NAME);
      SendMessage (hwndStatus, SB_SETTEXT, (WPARAM) 255 | 0, (LPARAM) (LPSTR) sStatusText);
    }
    
  switch (message)
    {
    case WM_CREATE:
      hInst = ((LPCREATESTRUCT) lParam)->hInstance;

      /*
       * Create EDIT control
       */
      hwndEdit = CreateWindowEx (WS_EX_STATICEDGE, "EDIT", NULL,
                   WS_CHILD | WS_VISIBLE | ES_LEFT | ES_NOHIDESEL
                   | ES_AUTOHSCROLL, 5, 19, 212, 18, hwnd,
                   (HMENU) ID_EDITCHILD, hInst, NULL);

      SendMessage (hwndEdit, EM_LIMITTEXT, MAX_PATH, 0L);
      SendMessage (hwndEdit, WM_SETFONT, (WPARAM) GetFont ("Arial", 16, FW_NORMAL), 0);

      /*
       * Create "Select" button
       */
      hwndButtonSelect = CreateWindowEx (0, "BUTTON", "...",
                       WS_VISIBLE | WS_CHILD,
                       222, 20, 18, 18,
                       hwnd, (HMENU) ID_SELECT_BUTTON, hInst,
                       NULL);
      SendMessage (hwndButtonSelect, WM_SETFONT,
           (WPARAM) GetFont ("Tahoma", 13, FW_NORMAL), 0);
      
      /*
       * Create radiobuttons
       */
      hwndRadio1 = CreateWindowEx (0, "BUTTON", "(A:)",
                 WS_VISIBLE | WS_CHILD | BS_RADIOBUTTON |
                 BS_LEFTTEXT, 10, 66, 40, 15, hwnd,
                 (HMENU) ID_RADIO_BUTTON1, hInst, NULL);
      hwndRadio2 = CreateWindowEx (0, "BUTTON", "(B:)",
                 WS_VISIBLE | WS_CHILD | BS_RADIOBUTTON |
                 BS_LEFTTEXT, 10, 83, 40, 15, hwnd,
                 (HMENU) ID_RADIO_BUTTON2, hInst, NULL);

      SendMessage (hwndRadio1, WM_SETFONT,
           (WPARAM) GetFont ("Tahoma", 13, FW_BOLD), 0);
      SendMessage (hwndRadio2, WM_SETFONT,
           (WPARAM) GetFont ("Tahoma", 13, FW_BOLD), 0);

      SendMessage (hwndRadio1, BM_SETCHECK, (WPARAM) BST_CHECKED, 0);
      SendMessage (hwndRadio2, BM_SETCHECK, (WPARAM) BST_UNCHECKED, 0);
      lstrcpy (sDestDrive, "A");

      /*
       * Create 'Info'-EDIT control
       */
      hwndEditInfo = CreateWindowEx (WS_EX_STATICEDGE, "EDIT", NULL, WS_CHILD | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_LEFT | ES_AUTOVSCROLL | ES_READONLY, 5, 109, 235, 70, hwnd, (HMENU) 0, hInst, NULL);
      
      SendMessage (hwndEditInfo, WM_SETFONT, (WPARAM) GetFont ("Courier", 8, FW_NORMAL), 0);

      /*
       * Create "Close" button
       */
      hwndButtonClose = CreateWindowEx (0, "BUTTON", "OK",
                      WS_VISIBLE | WS_CHILD |
                      BS_DEFPUSHBUTTON, 5, 109, 75, 25, hwnd,
                      (HMENU) ID_CLOSE_BUTTON, hInst, NULL);

      SendMessage (hwndButtonClose, WM_SETFONT,
           (WPARAM) GetFont ("Tahoma", 13, FW_NORMAL), 0);

      /*
       * Create "Run" button
       */
      hwndButtonRun = CreateWindowEx (0, "BUTTON", "Run",
                    WS_VISIBLE | WS_CHILD,
                    85, 109, 75, 25,
                    hwnd, (HMENU) ID_RUN_BUTTON, hInst, NULL);

      SendMessage (hwndButtonRun, WM_SETFONT,
           (WPARAM) GetFont ("Tahoma", 13, FW_NORMAL), 0);

      /*
       * Create "More Info" button
       */
      hwndButtonInfo = CreateWindowEx (0, "BUTTON", "More >>",
                     WS_VISIBLE | WS_CHILD,
                     165, 109, 75, 25,
                     hwnd, (HMENU) ID_INFO_BUTTON, hInst, NULL);

      SendMessage (hwndButtonInfo, WM_SETFONT,
           (WPARAM) GetFont ("Tahoma", 13, FW_NORMAL), 0);

      /* Create the STATUSBAR */
      hwndStatus = CreateWindowEx (0, STATUSCLASSNAME, NULL, WS_VISIBLE | WS_CHILD, 0, 0, 0, 0, hwnd, (HMENU) 0, hInst, NULL);
      SendMessage (hwndStatus, SB_SIMPLE, (WPARAM) (BOOL) TRUE, 0);
      SendMessage (hwndStatus, WM_SETFONT, (WPARAM) GetFont ("Tahoma", 13, FW_NORMAL), 0);
      wsprintf (sStatusText, "%s (Press F1 for help)", RES_PROGRAM_NAME);
      SendMessage (hwndStatus, SB_SETTEXT, (WPARAM) 255 | 0, (LPARAM) (LPSTR) sStatusText);

      /* Set focus on the EDIT control */
      SetFocus (hwndEdit);

      /*
       * Reset some strings
       */
      sFileName[0] = '\0';
      sFullFileName[0] = '\0';

      return 0;
      
    case WM_PAINT:
      hdc = BeginPaint (hwnd, &ps);
      SetBkMode (hdc, TRANSPARENT);
      SelectObject (hdc, GetFont ("Tahoma", 13, FW_NORMAL));

      /*
       * Write some text on the window
       */
      GetClientRect (hwnd, &rc);
      rc.left += 5;
      rc.top += 3;
      DrawText (hdc, sWndText1, lstrlen (sWndText1), &rc, DT_LEFT);
      rc.top += 47;
      DrawText (hdc, sWndText2, lstrlen (sWndText2), &rc, DT_LEFT);

      EndPaint (hwnd, &ps);
      break;

    case WM_SYSCOMMAND:
      switch (LOWORD (wParam))
        {
        case IDM_SYS_ABOUT:
          DialogBox (hInst, MAKEINTRESOURCE (IDD_ABOUTDIALOG), hwnd, (DLGPROC) DlgProcAbout);
          break;
        }
      break;
      
    case WM_COMMAND:
      if (lParam && HIWORD (wParam) == BN_CLICKED)
        {
          switch (LOWORD (wParam))
            {
            case ID_CLOSE_BUTTON:
              SendMessage (hwnd, WM_CLOSE, 0, 0);
              break;
            
            case ID_INFO_BUTTON:
              ShowWindow (hwndEditInfo, bInfoWindow = (bInfoWindow) ? FALSE : TRUE);
              GetWindowRect (hwnd, &rc);
              rc.right -= rc.left;
              if (bInfoWindow)
                {
                  MoveWindow (hwnd, rc.left, rc.top, rc.right, 257, TRUE);
                  MoveWindow (hwndButtonClose, 5, 184, 75, 25, TRUE);
                  MoveWindow (hwndButtonRun, 85, 184, 75, 25, TRUE);
                  MoveWindow (hwndButtonInfo, 165, 184, 75, 25, TRUE);
                  SetWindowText (hwndButtonInfo, "Less <<");
                  MoveWindow (hwndStatus, 0, 0, 0, 0, TRUE);
                }
              else
                {
                  MoveWindow (hwnd, rc.left, rc.top, rc.right, 182, TRUE);
                  MoveWindow (hwndButtonClose, 5, 109, 75, 25, TRUE);
                  MoveWindow (hwndButtonRun, 85, 109, 75, 25, TRUE);
                  MoveWindow (hwndButtonInfo, 165, 109, 75, 25, TRUE);
                  SetWindowText (hwndButtonInfo, "More >>");
                  MoveWindow (hwndStatus, 0, 0, 0, 0, TRUE);
                }
              SetFocus (hwndButtonInfo);
              break;
            
            case ID_RADIO_BUTTON1:
              SendMessage (hwndRadio1, BM_SETCHECK, (WPARAM) BST_CHECKED, 0);
              SendMessage (hwndRadio2, BM_SETCHECK, (WPARAM) BST_UNCHECKED, 0);
              lstrcpy (sDestDrive, "A");
              break;
            
            case ID_RADIO_BUTTON2:
              SendMessage (hwndRadio1, BM_SETCHECK, (WPARAM) BST_UNCHECKED, 0);
              SendMessage (hwndRadio2, BM_SETCHECK, (WPARAM) BST_CHECKED, 0);
              lstrcpy (sDestDrive, "B");
              break;

            case ID_RUN_BUTTON:
              SendMessage (hwnd, WM_COMMAND, IDA_FILE_RUN, 0);
              break;
              
            case ID_SELECT_BUTTON:
              SendMessage (hwnd, WM_COMMAND, IDA_FILE_SELECT, 0);
              break;
            }
        }
      
      switch (LOWORD (wParam))
        {
        case IDA_FILE_RUN:
          GetWindowText (hwndEdit, sFullFileName, GetWindowTextLength (hwndEdit) + 1);
          if (lstrcmp (sFullFileName, "") == 0)
            {
              MessageBox (hwnd, "No image file was selected!!!", RES_PROGRAM_NAME, MB_ICONWARNING);
              SetFocus (hwndEdit);
              return -1;
            }

          if (!DoesImageExist (sFullFileName))
            {
              MessageBox (hwnd, "The image file does not exist!!!", RES_PROGRAM_NAME, MB_ICONWARNING);
              SendMessage (hwndEdit, EM_SETSEL, 0, (LPARAM) lstrlen (sFullFileName));
              SetFocus (hwndEdit);
              return -1;
            }
          SendMessage (hwndStatus, SB_SETTEXT, (WPARAM) 255 | 0, (LPARAM) (LPSTR) "Please wait");

          EnableWindow (hwndEdit, FALSE);
          EnableWindow (hwndButtonSelect, FALSE);
          EnableWindow (hwndRadio1, FALSE);
          EnableWindow (hwndRadio2, FALSE);
          EnableWindow (hwndButtonClose, FALSE);
          EnableWindow (hwndButtonRun, FALSE);
  
          bActive = TRUE;
          iTimer = 0;
          SetTimer (hwnd, ID_TIMER, 1500, (TIMERPROC) NULL);
          RunRawrite (&pi, hwndEditInfo, sExeDir, sFullFileName, sDestDrive);
          break;
          
        case IDA_FILE_SELECT:
          sFileName[0] = '\0';
          sFullFileName[0] = '\0';
          if (!SelectFile (hwnd, sFileName, sCurrentDir) || lstrcmp (sFileName, "") == 0)
            return -1;
          GetCurrentDirectory (MAX_PATH, sCurrentDir);
          wsprintf (sFullFileName, "%s\\%s", sCurrentDir, sFileName);
          SetWindowText (hwndEdit, sFullFileName);
          break;
        }
      break;
      
    case WM_HELP:
      switch (((LPHELPINFO) lParam)->iCtrlId)
      {
      case ID_EDITCHILD:
        WinHelp (hwnd, sHelpFile, HELP_CONTEXTPOPUP, 2000);
        break;

      case ID_RADIO_BUTTON1:
      case ID_RADIO_BUTTON2:
        WinHelp (hwnd, sHelpFile, HELP_CONTEXTPOPUP, 2001);
        break;

      default:
        WinHelp (hwnd, sHelpFile, HELP_FINDER, 0);
        return 0;
      }
      break;

    case WM_TIMER:
      switch (wParam)
        {
        case ID_TIMER:
          if (bActive)
            {
              lstrcpy (sStatusText, "Please wait");
              iTimer++;
              for (iCount = 0; iCount < iTimer; iCount++)
                lstrcat (sStatusText, ".");
              SendMessage (hwndStatus, SB_SETTEXT, (WPARAM) 255 | 0, (LPARAM) (LPSTR) sStatusText);
            }
          break;
        }
      break;
      
    case WM_CLOSE:
      if (bActive)
        iAnswer = MessageBox (hwnd, "RAWRITE is processing...!\nDo you really want to quit?", RES_PROGRAM_NAME, MB_YESNO | MB_ICONWARNING);
      if (!bActive || iAnswer == IDYES)
        DestroyWindow (hwnd);
      return 0;
      
    case WM_DESTROY:
      PostQuitMessage (0);
      return 0;
    }
  return DefWindowProc (hwnd, message, wParam, lParam);
}
